package fwadmin;

/*

  JFwadmin v 0.82

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import javax.swing.table.*;
import java.awt.Color;
import javax.swing.*;
import java.awt.event.*;

public class DefaultPolicy
{
  private static String defaultInput;
  private static String defaultOutput;
  private static String defaultForward;
  protected static boolean defaultInputNotApplied;
  protected static boolean defaultOutputNotApplied;
  protected static boolean defaultForwardNotApplied;

  protected static final String firewallDenyPolicy = "DENY";
  protected static final String firewallAcceptPolicy = "ACCEPT";
  protected static final String firewallForwardPolicy = "FORWARD";
  protected static final String firewallMasqPolicy = "MASQ";

  private static DefaultPolicy m_instance;
	private static Color oddRowColor = new Color(0xD6, 0xDD, 0xF3);
  private DefaultPolicy() {}  
public static DefaultTableCellRenderer getCommentRenderer()
{
	DefaultTableCellRenderer commentRenderer = new DefaultTableCellRenderer()
	{
		public java.awt.Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			if (value != null)
			{
				String sval = (String) value;
				if (sval.indexOf("not applied") != -1)
					setBackground(Color.lightGray);
				else
					setBackground(Color.white);
				setText(sval);
			}
			return this;
		}
	};
	return commentRenderer;
}
  public static String getForwardDefaultPolicy()
  {
	return defaultForward;
  }  
  public static String getInputDefaultPolicy()
  {
	return defaultInput;
  }      
  public static DefaultPolicy getInstance()
  {
	if (m_instance == null) m_instance = new DefaultPolicy();
	return m_instance;
  }  
  public static String getOutputDefaultPolicy()
  {
	return defaultOutput;
  }  
public static DefaultTableCellRenderer getPolicyRenderer()
{
	// Show nice (?) colors for policies.
	DefaultTableCellRenderer policyRenderer = new DefaultTableCellRenderer()
	{
		public java.awt.Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			String sval = (String) value;
			if (sval.equals(DefaultPolicy.firewallDenyPolicy))
			{
				setBackground(Color.red.brighter());
				setForeground(Color.black);
			}
			else
			if (sval.equals(DefaultPolicy.firewallAcceptPolicy))
			{
				setBackground(Color.green.brighter());
				setForeground(Color.black);
			}
			else
			if (sval.equals(DefaultPolicy.firewallForwardPolicy))
			{
				setBackground(Color.blue.darker().darker());
				setForeground(Color.white);
			}
			else
			if (sval.equals(DefaultPolicy.firewallMasqPolicy))
			{
				setBackground(Color.blue.brighter());
				setForeground(Color.white);
			}
			else
			{
				setBackground(Color.gray);
				setForeground(Color.black);
			}
			if (isSelected)
			{
				setBackground(getBackground().brighter());
				setForeground(getForeground().brighter());
			}
			setText(sval);
			return this;
		}
	};
	return policyRenderer;
}
public static DefaultTableCellRenderer getProtocolRenderer()
{
	// Allow to put tooltips on cells.
	DefaultTableCellRenderer protocolRenderer = new DefaultTableCellRenderer()
	{
		public java.awt.Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			if (value != null)
			{
				if (value.equals("all"))
				{
					setBackground(Color.white);
					setForeground(Color.darkGray);
				}
				else
				if (value.equals("tcp"))
				{
					setBackground(Color.lightGray);
					setForeground(Color.black);
				}
				else
				if (value.equals("udp"))
				{
					setBackground(Color.darkGray);
					setForeground(Color.white);
				}
				else
				if (value.equals("icmp"))
				{
					setBackground(Color.black);
					setForeground(Color.white);
				}
				String sval = (String) value;
				setText(sval);
			}
			return this;
		}
	};
	return protocolRenderer;
}
public static DefaultTableCellRenderer getStringRenderer()
{
	// Allow to put tooltips on cells.
	DefaultTableCellRenderer stringRenderer = new DefaultTableCellRenderer()
	{
		public java.awt.Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			if (isSelected)
			{
				setBackground(Color.yellow);
				setForeground(Color.black);
			}
			else
			{
				if(row%2 == 1)
					setBackground(oddRowColor);
				else
					setBackground(Color.white);
				setForeground(Color.darkGray);
			}
			
			if (value != null)
			{
				String sval = (String) value;
				setText(sval);
				setToolTipText(sval);
			}
			return this;
		}
	};
	return stringRenderer;
}
public static DefaultTableCellRenderer getWhiteBackgroundRenderer()
{
	DefaultTableCellRenderer whiteBackgroundRenderer = new DefaultTableCellRenderer()
	{
		public java.awt.Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			if (isSelected)
			{
				setBackground(Color.yellow);
				setForeground(Color.black);
			}
			else
			{
				if(row%2 == 1)
					setBackground(oddRowColor);
				else
					setBackground(Color.white);
				setForeground(Color.darkGray);
			}

			if (value != null)
			{
				String sval = (String) value;
				setText(sval);
			}
			return this;
		}
	};
	return whiteBackgroundRenderer;
}
  public static void setForwardDefaultPolicy(String p)
  {
	if (defaultForward != null) defaultForwardNotApplied = ! defaultForward.equals(p);
	defaultForward = p;
  }  
  public static void setInputDefaultPolicy(String p)
  {
	if (defaultInput != null) defaultInputNotApplied = ! defaultInput.equals(p);
	defaultInput = p;
  }  
  public static void setOutputDefaultPolicy(String p)
  {
	if (defaultOutput != null) defaultOutputNotApplied = ! defaultOutput.equals(p);
	defaultOutput = p;
  }  
}
