package fwadmin;

/*

  JFwadmin v 0.82

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.TreeSet;
import java.util.Iterator;

public class ForwardServicesPanel extends JPanel implements UpdateWindow, ActionListener
{
  String[] columnsNames = {"Forward services / port(s)", "Interface", "Protocol", "Source address", "Source mask", "Destination address", "Destination mask", "Policy", "Comment"};
  Object[][] data = {};
  Object[][] old_data = {};
  JTable m_tableView = null;
  JScrollPane m_scrollPane = null;


  private javax.swing.JPopupMenu m_jPopupMenu;
	private javax.swing.JMenuItem m_jMenuItemDeselect;
	private javax.swing.JMenuItem m_jMenuItemRemove;
	private javax.swing.JMenuItem m_jMenuItemAdd;

  public ForwardServicesPanel()
  {
	this.setLayout(new BorderLayout());
	doDisplayTable();
  }    
public void actionPerformed(ActionEvent e)
{
	if (m_tableView.getSelectedRow() == -1) return;
	
	if (e.getSource().equals(m_jMenuItemDeselect))
	{
		m_tableView.clearSelection();
		return;
	}

	if (e.getSource().equals(m_jMenuItemRemove))
	{
		Rule selectedRule = (Rule)data[m_tableView.getSelectedRow()][columnsNames.length];
		Rules.removeRule(selectedRule);
		JFwadmin.getInstance().refreshPanels();
		return;
	}

	if (e.getSource().equals(m_jMenuItemAdd))
	{
		JFwadmin.getInstance().new_forward_serviceMenuItem_ActionEvents();
	}
}
  public void doDisplayTable()
  {
	// Create a model of the data.
	TableModel dataModel = new AbstractTableModel() {
	public int getColumnCount() { return columnsNames.length; }
	public int getRowCount() { return data.length;}
	public Object getValueAt(int row, int col) {return data[row][col];}
	public String getColumnName(int column) {return columnsNames[column];}
	public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
	public boolean isCellEditable(int row, int col) {return false;}
	public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
	};

	// Create the table
	m_tableView = new JTable(dataModel);
	m_tableView .setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	m_tableView.setRowHeight(InputServicesPanel.m_rowHeightInitialValue);
  	m_tableView.setShowGrid(false);
  	m_tableView.addMouseListener(new MouseAdapter() {
	public void mouseClicked(MouseEvent e)
	{
		if (m_tableView.getSelectedRow() == -1) return;

		if ((e.getModifiers() & e.BUTTON3_MASK) != 0)
		{
		  getJPopupMenu().show(m_tableView, e.getX(), e.getY());
		}
	}
	});

  	m_scrollPane = new JScrollPane(m_tableView);
	this.add("Center", m_scrollPane);
	m_tableView.setModel(dataModel);
	for(int i=0; i < columnsNames.length; i++)
	{
		if (columnsNames[i].equals("Policy"))
			m_tableView.getColumn(columnsNames[i]).setCellRenderer(DefaultPolicy.getPolicyRenderer());
		else
		if (columnsNames[i].equals("Protocol"))
			m_tableView.getColumn(columnsNames[i]).setCellRenderer(DefaultPolicy.getProtocolRenderer());
		else
			m_tableView.getColumn(columnsNames[i]).setCellRenderer(DefaultPolicy.getStringRenderer());
	}
	m_tableView.getColumn(columnsNames[0]).setPreferredWidth(100);
	m_tableView.getColumn("Policy").setPreferredWidth(30);
	m_tableView.getColumn("Protocol").setPreferredWidth(20);
	m_tableView.getColumn("Interface").setPreferredWidth(20);
	m_tableView.sizeColumnsToFit(-1);
  }          
/**
 * Insert the method's description here.
 * Creation date: (17/02/00 02:39:11)
 * @return javax.swing.JPopupMenu
 */
public JPopupMenu getJPopupMenu()
{
	if (m_jPopupMenu == null)
	{
		m_jPopupMenu = new JPopupMenu("Choose Action");
		m_jPopupMenu.setLabel("Choose Action");
		m_jMenuItemDeselect = new JMenuItem("Deselect");
		m_jMenuItemDeselect.addActionListener(this);
		m_jPopupMenu.add(m_jMenuItemDeselect);
		m_jMenuItemRemove = new JMenuItem("Remove");
		m_jMenuItemRemove.addActionListener(this);
		m_jPopupMenu.add(m_jMenuItemRemove);
		m_jMenuItemAdd = new JMenuItem("Add new service");
		m_jMenuItemAdd.addActionListener(this);
		m_jPopupMenu.add(m_jMenuItemAdd);
	}
	return m_jPopupMenu;
}
  public void updateWindow()
  {
	int index = 0;
	Iterator it = Rules.rulesTreeSet.iterator();
	while(it.hasNext())
	{
	  Rule r = (Rule) it.next();
	  if(! r.isForwardRule()) continue;
	  index++;
	}
	
	data = new Object[index][columnsNames.length + 1];
	
	it = Rules.rulesTreeSet.iterator();
	index = 0;
	while(it.hasNext())
	{
	  Rule r = (Rule) it.next();
	  if(! r.isForwardRule()) continue;
	  data[index][0] = r.getDestinationPortsStringForDisplay();
	  data[index][1] = r.m_interfaceName;
	  data[index][2] = r.getProtocolString();
	  data[index][3] = r.m_sourceAddress.toString();
	  data[index][4] = r.m_sourceMask.toString();
	  data[index][5] = r.m_destinationAddress.toString();
	  data[index][6] = r.m_destinationMask.toString();
	  data[index][7] = r.m_policy;
	  data[index][columnsNames.length] = r;
	  String Comment = "";
	  if ((r.m_flags & Linux22.FLAG_LOGGING) > 0)
	  {
		Comment += "logged";
	  }
	  if (! r.m_isApplied)
	  {
		Comment += "not applied";
	  }
	  data[index][8] = Comment;
	  index++;
	}

	// Avoid table blinking : verify data changed, and if not, don't update table.
	boolean differenceEncountered = false;
	if (data.length != old_data.length) differenceEncountered = true;
	if (! differenceEncountered)
	{
	  for(int i=0; i < data.length; i++)
	  {
		if (data[i].length != old_data[i].length) differenceEncountered = true;
		if (differenceEncountered) break;

		for (int j=0; j < data[i].length; j++)
		{
		  if (!(data[i][j].equals(old_data[i][j]))) differenceEncountered = true;
		  if (differenceEncountered) break;
		}
	  }
	}
	// No difference between new and old data, so nothing to do and return.
	if (! differenceEncountered) return;

	old_data = data;

	m_tableView.tableChanged(new TableModelEvent(m_tableView.getModel()));
	validate();
  }                            
}
