package fwadmin;

/*

  JFwadmin v 0.82

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.util.*;
import java.io.*;

public class Linux22
{
	protected static Hashtable port2serviceHashtable;
	protected static Hashtable service2portHashtable;
	protected static Vector servicesVector;
	protected static Hashtable m_varsHashtable;


	protected static PrintWriter m_printWriter;

	static final int FLAG_LOGGING = 1;	   /* This flag indicates logging.  */
	static final int FLAG_TCP_SYN = 2;	   /* This flag indicates syn packets only admittance */
	static final int FLAG2_NO_TCP_SYN = 0x40;	   /* This flag indicates reverse on tcp syn permission */

	static final int IPPROTO_IP = 0;	   /* Dummy protocol for TCP.  */
	static final int IPPROTO_ICMP = 1;	   /* Internet Control Message Protocol.  */
	static final int IPPROTO_TCP = 6;	   /* Transmission Control Protocol.  */
	static final int IPPROTO_UDP = 17;	   /* User Datagram Protocol.  */
/**
 * LinuxIPFWConstants constructor comment.
 */
private Linux22() {
	super();
}
public static void applyConfiguration() throws Exception
{
	applyRules();
	applyMasqModules();
	applyPortForwarding();
}
public static void applyMasqModules() throws Exception
{
	// Now execute commands to handle modules
	for (int i = 0; i < MasqModule.masqModulesVector.size(); i++)
	{
		MasqModule mm = (MasqModule) MasqModule.masqModulesVector.elementAt(i);
		if (m_printWriter != null)
		{
			if (mm.mb_enabled)
				saveOrExec(new String[] {JFwadmin.getResource("insmod_bin"), mm.mstr_name});
		}
		else
		if (mm.mb_enabled != mm.mb_systemEnabled)
		{
			try
			{
				if (mm.mb_enabled)
					saveOrExec(new String[] {JFwadmin.getResource("insmod_bin"), mm.mstr_name});
				else
					saveOrExec(new String[] {JFwadmin.getResource("rmmod_bin"), mm.mstr_name});
			}
			catch (Exception e)
			{
			}
		}
	}
}
public static void applyPortForwarding() throws Exception
{
	if (! JFwadmin.getInstance().getPortForwardingEnabled()) return;
	
	// Flush port forward rules
	saveOrExec(new String[] {JFwadmin.getResource("ipmasqadm_bin"), JFwadmin.getResource("ipmasqadm_mode"), "-f"});
	// Now execute commands to establish rules
	for (int i=0; i < PortForwardRule.portForwardRulesVector.size(); i++)
	{
	  PortForwardRule pfr = (PortForwardRule)PortForwardRule.portForwardRulesVector.elementAt(i);
	  saveOrExec(new String [] {JFwadmin.getResource("ipmasqadm_bin"), JFwadmin.getResource("ipmasqadm_mode"), "-a", "-P", pfr.mstr_protocol, "-L", pfr.m_localIpAddress.toString(), "" + pfr.mi_localPort, "-R", pfr.m_remoteIpAddress.toString(), "" + pfr.mi_remotePort});
	}
}
public static void applyRules() throws Exception
{
	// Flush input rules
	saveOrExec(new String[] {JFwadmin.getResource("ipchains_bin"), "-F", "input"});
	saveOrExec(new String[] {JFwadmin.getResource("ipchains_bin"), "-P", "input", DefaultPolicy.getInputDefaultPolicy()});
	// Flush output rules
	saveOrExec(new String[] {JFwadmin.getResource("ipchains_bin"), "-F", "output"});
	saveOrExec(new String[] {JFwadmin.getResource("ipchains_bin"), "-P", "output", DefaultPolicy.getOutputDefaultPolicy()});
	// Flush output rules
	saveOrExec(new String[] {JFwadmin.getResource("ipchains_bin"), "-F", "forward"});
	saveOrExec(new String[] {JFwadmin.getResource("ipchains_bin"), "-P", "forward", DefaultPolicy.getForwardDefaultPolicy()});

	// Now execute commands to establish rules
	Iterator it = Rules.rulesTreeSet.iterator();
	while(it.hasNext())
	{
	  Rule r = (Rule) it.next();
	  Vector v_args = new Vector();
	  v_args.addElement(JFwadmin.getResource("ipchains_bin"));
	  v_args.addElement("-A");
	  v_args.addElement(r.m_ruleType);
	  v_args.addElement("-p");
	  v_args.addElement(r.getProtocolString());
	  v_args.addElement("-j");
	  v_args.addElement(r.m_policy);
	  v_args.addElement("-i");
	  v_args.addElement(r.m_interfaceName);
	  v_args.addElement("-s");
	  v_args.addElement(r.m_sourceAddress.toString() + r.m_sourceMask.getMaskStringForScript());
	  if (r.m_protocol != IPPROTO_IP)
	  {
	  	v_args.addElement(r.getSourcePortsString());
	  }
	  v_args.addElement("-d");
	  v_args.addElement(r.m_destinationAddress.toString() + r.m_destinationMask.getMaskStringForScript());
	  if (r.m_protocol != IPPROTO_IP)
	  {
	  	v_args.addElement(r.getDestinationPortsString());
	  }
	  if ((r.m_flags & Linux22.FLAG_LOGGING) > 0) v_args.addElement("-l");
	  if (((r.m_flags & Linux22.FLAG_TCP_SYN) > 0) && ((r.m_flags2 & Linux22.FLAG2_NO_TCP_SYN) > 0))
	  {
		  v_args.addElement("!");
		  v_args.addElement("-y");
	  }

	  String [] args = new String[v_args.size()];
	  for (int j=0; j < v_args.size(); j++)
		args[j] = (String)v_args.elementAt(j);

	  saveOrExec(args);
	}
}
public static Process exec(String [] args) throws Exception
{
  Process p = Runtime.getRuntime().exec(args, new String[]{"LC_ALL=en_EN"});
  return p;
}
public static int getProtocolConstant(String prot)
{
  if(prot.equals("tcp")) return Linux22.IPPROTO_TCP;
  if(prot.equals("udp")) return Linux22.IPPROTO_UDP;
  if(prot.equals("icmp")) return Linux22.IPPROTO_ICMP;
  if(prot.equals("all")) return Linux22.IPPROTO_IP;
  return 0;
}
public static String getServiceName(int port, String protocolType)
{
  String theKey = "" + port + "/" + protocolType;
  String theElement = (String)port2serviceHashtable.get(theKey);
  if (theElement != null) return theElement;

  return "" + port;
}
public static int getServicePortNumber(String name, String protocolType)
{
  try
  {
	int i_port = Integer.parseInt(name);
	return i_port;
  }
  catch(NumberFormatException nfe)
  {}
  String theKey = (new StringTokenizer(name)).nextToken() + "/" + protocolType;
  Integer theInteger = (Integer)service2portHashtable.get(theKey);
  if (theInteger == null) return 0;
  return theInteger.intValue();
}
private static void initServicesHashtables()
{
	port2serviceHashtable = new Hashtable();
	service2portHashtable = new Hashtable();
	servicesVector = new Vector();
	try
	{
		FileReader fr_services = new FileReader("/etc/services");
		//FileReader fr_services = new FileReader("C:\\proc\\net\\services");
		BufferedReader bf = new BufferedReader(fr_services);
		while (true)
		{
			String line = bf.readLine();
			if (line == null)
				break;
			try
			{
				StringTokenizer st = new StringTokenizer(line, " \t\r\n");
				while (st.hasMoreElements())
				{
					String name = st.nextToken();
					if (name.startsWith("#"))
						break; // Comment
					String port = st.nextToken();
					port2serviceHashtable.put(port, name);
					StringTokenizer st2 = new StringTokenizer(port, "/");
					Integer portNumber = new Integer(st2.nextToken());
					String prot = st2.nextToken();
					service2portHashtable.put(name + "/" + prot, portNumber);
					boolean inserted = false;
					name += " (" + portNumber + ")";
					for (int i = 0; i < servicesVector.size(); i++)
					{
						int cmp = name.compareTo((String) servicesVector.elementAt(i));
						if (cmp == 0)
						{
							// Service name already present
							inserted = true;
							break;
						}
						if (cmp < 0)
						{
							servicesVector.insertElementAt(name, i);
							inserted = true;
							break;
						}
					}
					if (!inserted)
						servicesVector.addElement(name);
					break;
				}
			}
			catch (Exception e)
			{
				// Ignore this service
			}
		}
	}
	catch (java.io.FileNotFoundException fnfe)
	{
		System.out.println("can't open file " + fnfe.toString());
	}
	catch (java.io.IOException ioe)
	{
		System.out.println(ioe.toString());
	}
}
public static void readFirewallRules()
{
	// Now read chains default policy
	try
	{
		FileReader fr_policy = new FileReader(JFwadmin.getResource("ip_fwnames_file"));
		BufferedReader bf = new BufferedReader(fr_policy);
		while (true)
		{
			String line = bf.readLine();
			if (line == null)
				break;
			if (line.compareTo("") > 0)
			{
				StringTokenizer st = new StringTokenizer(line);
				String chain = st.nextToken();
				if (chain.equals("input"))
				{
					DefaultPolicy.setInputDefaultPolicy(st.nextToken());
					DefaultPolicy.defaultInputNotApplied = false;
				}
				if (chain.equals("output"))
				{
					DefaultPolicy.setOutputDefaultPolicy(st.nextToken());
					DefaultPolicy.defaultOutputNotApplied = false;
				}
				if (chain.equals("forward"))
				{
					DefaultPolicy.setForwardDefaultPolicy(st.nextToken());
					DefaultPolicy.defaultForwardNotApplied = false;
				}
			}
		}
	}
	catch (java.io.FileNotFoundException fnfe)
	{
		System.out.println("can't open file " + fnfe.toString());
		System.out.println("if " + JFwadmin.getResource("ip_fwnames_file") + " does not exist, your system is probably not configured properly for firewalling.");
	}
	catch (java.io.IOException ioe)
	{
		System.out.println(ioe.toString());
	}

	// Now read firewall rules
	try
	{
		FileReader fr_route = new FileReader(JFwadmin.getResource("ip_fwchains_file"));
		//FileReader fr_route = new FileReader("C:\\proc\\net\\ip_fwchains");
		BufferedReader bf = new BufferedReader(fr_route);
		int lineNumber = 0;
		while (true)
		{
			String rawRule = bf.readLine();
			if (rawRule == null)
				break;
			if (rawRule.compareTo("") > 0)
			{
				StringTokenizer st = new StringTokenizer(rawRule);
				Rule r = new Rule(rawRule);
				r.m_isApplied = true;
				Rules.rulesTreeSet.add(r);
			}
			lineNumber++;
		}
	}
	catch (java.io.FileNotFoundException fnfe)
	{
		System.out.println("can't open file " + fnfe.toString());
		System.out.println("file " + JFwadmin.getResource("ip_fwchains_file") + " does not exist, your system is probably not configured properly for firewalling.");
	}
	catch (java.io.IOException ioe)
	{
		System.out.println(ioe.toString());
	}
}
public static void readMasqModules()
{
	try
	{
		Process p = exec(new String[]{JFwadmin.getResource("uname_bin"), "-r"});
		BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
		String kernelVersion = bf.readLine();
		p = exec(new String[]{JFwadmin.getResource("ls_bin"), JFwadmin.getResource("modules_path_1") + kernelVersion + JFwadmin.getResource("modules_path_2")});
		bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
		while(true)
		{
			String line = bf.readLine();
			if (line == null) break;
			StringTokenizer st = new StringTokenizer(line);
			while(st.hasMoreTokens())
			{
				StringTokenizer st2 = new StringTokenizer(st.nextToken(), ".");
				String module = st2.nextToken();
				if (module.startsWith("ip_masq"))
				{
					MasqModule mm = new MasqModule(module, false);
					MasqModule.masqModulesVector.add(mm );
				}
			}
		}
	}
	catch(Exception e)
	{
		System.out.println("Exception : " + e.toString());
	}


	try
	{
		FileReader fr = new FileReader(JFwadmin.getResource("modules_proc_file"));
		BufferedReader bf = new BufferedReader(fr);
		while(true)
		{
			String line = bf.readLine();
			if (line == null) break;
			StringTokenizer st = new StringTokenizer(line);
			String module = st.nextToken();
			if (module.startsWith("ip_masq"))
			{
				MasqModule mm = (MasqModule) MasqModule.getMasqModule(module);
				mm.mb_enabled = true;
				mm.mb_systemEnabled = true;
			}
		}
	}
	catch(Exception e)
	{
		System.out.println("Exception : " + e.toString());
	}
}
public static void readNetConfig()
{
  // Read interfaces.
  try
  {
	Process p = exec(new String[] {JFwadmin.getResource("ifconfig_bin")});
	BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
	int lineNumber = 0;
	int lineNumberBeginInterfaceDefinition = 0;
	String if_name = null;
	while(true)
	{
	  String line = bf.readLine(); lineNumber++;
	  if(line == null) break;
	  //System.out.println("read : " + line);
	  if ((! line.startsWith(" ")) && (! line.startsWith("\t")))
	  {
		lineNumberBeginInterfaceDefinition = lineNumber;
		StringTokenizer st = new StringTokenizer(line);
		if (st.hasMoreTokens()) if_name = st.nextToken();
	  }
	  if (lineNumber == (lineNumberBeginInterfaceDefinition + 1))
	  {
		StringTokenizer st = new StringTokenizer(line, " :");
		String if_addr = null;
		String if_mask = null;
		int tokenIndex = 0;
		while(st.hasMoreTokens())
		{
		  String token = st.nextToken();
		  if (token.equals("addr")) if_addr = st.nextToken();
		  if (token.equals("Mask")) if_mask = st.nextToken();
		  if (if_mask != null) new IPInterface(if_name, if_addr, if_mask);
		}
	  }
	}
  }
  catch(Exception e)
  {
	System.out.println("Problem executing " + JFwadmin.getResource("ifconfig_bin") + " (?) : " +e.toString());
	System.out.println("This may be a problem with your local settings.");
  }

  // Now read routing.
  try
  {
	FileReader fr_route = new FileReader(JFwadmin.getResource("route_proc_file"));
	BufferedReader bf = new BufferedReader(fr_route);
	int lineNumber = 0;
	int lineNumberBeginInterfaceDefinition = 0;
	while(true)
	{
	  String line = bf.readLine(); lineNumber++;
	  if(line == null) break;
	  if (lineNumber > 1)
	  {
		StringTokenizer st = new StringTokenizer(line, " \t");
		  String if_name = st.nextToken();
		  String destination = st.nextToken();
		  String gateway = st.nextToken();
		  st.nextToken();
		  st.nextToken();
		  st.nextToken();
		  st.nextToken();
		  String genmask = st.nextToken();
		  IPInterface.getInterface(if_name).addRoute(destination, gateway, genmask);
	  }
	}
  }
  catch(Exception e)
  {
	System.out.println("Problem reading " + JFwadmin.getResource("route_proc_file") + " : " + e.toString());
	e.printStackTrace();
  }

  initServicesHashtables();
}
public static void readPortForwarding()
{
	try
	{
		FileReader fr = new FileReader(JFwadmin.getResource("portfw_proc_file"));
		BufferedReader bf = new BufferedReader(fr);
		// Skip header line
		bf.readLine();
		while(true)
		{
			String line = bf.readLine();
			if (line == null) break;
			StringTokenizer st = new StringTokenizer(line);
			
			String protocol = st.nextToken();
			protocol = protocol.toLowerCase();
			long la = Long.parseLong(st.nextToken(), 0x10);
			int lp = Integer.parseInt(st.nextToken());
			
			st.nextToken(); // sould be ">", ignored.
			
			long ra = Long.parseLong(st.nextToken(), 0x10);
			int rp = Integer.parseInt(st.nextToken());

			PortForwardRule pfr = new PortForwardRule(protocol, la, lp, ra, rp);
			PortForwardRule.portForwardRulesVector.addElement(pfr);
			pfr.mb_isApplied = true;
		}

		// Test if the ipmasqadm bin is installed.
		saveOrExec(new String[] {JFwadmin.getResource("ipmasqadm_bin"), JFwadmin.getResource("ipmasqadm_mode"), "-l"});
	}
	catch(Exception e)
	{
		JFwadmin.getInstance().disablePortForwarding();
		//System.out.println("Your system seems not configured to handle port forwarding (ipmasqadm portfw).");
	}
}
public static void restoreFirewall(javax.swing.JFileChooser chooser) throws Exception
{
	String error_string = "There was an error.";
	// Flush input rules
	Process p = Linux22.exec(new String[] {JFwadmin.getResource("ipchains_bin"), "-F", "input"});
	p.waitFor();
	int res = p.exitValue();
	if (res != 0) throw new Exception(error_string);
	// Flush output rules
	p = Linux22.exec(new String[] {JFwadmin.getResource("ipchains_bin"), "-F", "output"});
	p.waitFor();
	res = p.exitValue();
	if (res != 0) throw new Exception(error_string);
	// Flush output rules
	p = Linux22.exec(new String[] {JFwadmin.getResource("ipchains_bin"), "-F", "forward"});
	p.waitFor();
	res = p.exitValue();
	if (res != 0) throw new Exception(error_string);

	BufferedReader br = new BufferedReader(new FileReader(chooser.getSelectedFile().getAbsolutePath()));
	p = Linux22.exec(new String[] {JFwadmin.getResource("ipchains_restore_bin")});
	PrintWriter pw = new PrintWriter(new OutputStreamWriter(p.getOutputStream()));
	while(true)
	{
	  String line = br.readLine();
	  if(line == null) break;
	  pw.println(line);
	}
	pw.flush();
	pw.close();
	p.waitFor();
	res = p.exitValue();
	if (res != 0) throw new Exception("Process exit value is not 0");
}
public static void saveBootScript(PrintWriter pw) throws Exception
{
  m_printWriter = pw;
  writeScriptInitVars(pw);
  applyConfiguration();
  pw.flush();
  pw.close();
  m_printWriter = null;
}
public static void saveFirewall(javax.swing.JFileChooser chooser) throws Exception
{
	PrintWriter pw = new PrintWriter(new FileWriter(chooser.getSelectedFile().getAbsolutePath()));
	Process p = Linux22.exec(new String[] {JFwadmin.getResource("ipchains_save_bin")});
	BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
	while(true)
	{
	  String line = bf.readLine();
	  if(line == null) break;
	  pw.println(line);
	}
	p.waitFor();
	int res = p.exitValue();
	pw.flush();
	pw.close();

	if (res != 0) throw new Exception("Process exit value is not 0");
}
public static void saveOrExec(String [] args) throws Exception
{
  if (m_printWriter != null)
  {
	for (int i=0; i < args.length; i++)
	{
	  String toPrint = args[i];
	  if (m_varsHashtable.get(toPrint) != null) toPrint = "$" + m_varsHashtable.get(toPrint);
	  m_printWriter.print(toPrint + ((i<args.length-1) ? " " : ""));
	}
	m_printWriter.println();
	return;
  }

  Process p = exec(args);
  p.waitFor();
  int res = p.exitValue();
  if (res != 0) throw new Exception("There was an error.");
}
public static void setupScriptVar(String varname, String varvalue, PrintWriter pw)
{
  varname = varname.toUpperCase();
  m_varsHashtable.put(varvalue, varname);
  m_printWriter.println(varname + "=" + varvalue);
}
public static TreeSet sortRules(TreeSet from)
{
	TreeSet to = new TreeSet(new RuleComparator());
	to.addAll(from);	
	return to;
}
public static void writeScriptInitVars(PrintWriter pw)
{
  m_printWriter.println("#!/bin/sh");
  m_printWriter.println("# Firewall Script created " + (new Date()).toString());
  m_printWriter.println("# Script generator : " + JFwadmin.getApplicationName() + " " + JFwadmin.getApplicationVersion());
  m_printWriter.println("#");
  m_printWriter.println("# Make language english for parsing command results");
  m_printWriter.println("LC_ALL=en_EN");
  m_printWriter.println("# Set up vars");

  m_varsHashtable = new Hashtable();

  for (Enumeration e = IPInterface.interfacesHashtable.elements(); e.hasMoreElements(); )
  {
	IPInterface ipi = (IPInterface)e.nextElement();
	setupScriptVar(ipi.getName() + "_IF", ipi.getName(), pw);

	// Automatic determination of IP address within the script
	m_printWriter.println(ipi.getName().toUpperCase() + "_IP=`" + JFwadmin.getResource("ifconfig_bin") + " " + ipi.getName() + JFwadmin.getResource("if_ip_addr_sed"));
	m_varsHashtable.put(ipi.getAddress().toString(), (ipi.getName() + "_IP").toUpperCase());

	String varvalue = (new IPAddress(ipi.getAddress().getLong() & ipi.getMask().getLong())).toString();
	varvalue += "/" + (new IPMask(ipi.getMask().getLong())).toString();
	setupScriptVar(ipi.getName() + "_NET", varvalue, pw);
  }

  // Remove "anywhere" networks
  String anywhereNet = JFwadmin.getResource("anywhere_network");
  while(true)
  {
	Object o = m_varsHashtable.get(anywhereNet);
	if (o != null)
	  m_varsHashtable.remove(anywhereNet);
	else
	  break;
  }
  setupScriptVar("ANYWHERE", anywhereNet, pw);
  setupScriptVar("PORTS", JFwadmin.getResource("any_port"), pw);

  m_printWriter.println("# Remove used Masquerading Modules");
  m_printWriter.println("cat " + JFwadmin.getResource("modules_proc_file") + " | grep ip_masq > /tmp/ipmapp$$");
  m_printWriter.println("while read LINE");
  m_printWriter.println("do");
  m_printWriter.println(" set $LINE none none none");
  m_printWriter.println(" if [ \"$1\" != \"none\" ]");
  m_printWriter.println(" then");
  m_printWriter.println("  " + JFwadmin.getResource("rmmod_bin") + " $1");
  m_printWriter.println(" fi");
  m_printWriter.println("done < /tmp/ipmapp$$");
  m_printWriter.println("rm -f /tmp/ipmapp$$");
  m_printWriter.println();
  m_printWriter.println("# Here are the rules");
}
}
