package fwadmin;

/*

  JFwadmin v 0.82

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.util.StringTokenizer;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Iterator;

public class Rule extends Object
{
	protected String m_ruleType;
	protected IPAddress m_sourceAddress;
	protected IPMask m_sourceMask;
	protected IPAddress m_destinationAddress;
	protected IPMask m_destinationMask;
	protected String m_interfaceName;
	protected int  m_flags;
	protected int  m_flags2;
	protected String  m_policy;
	protected int  m_protocol;
	protected int  m_sourceLowPort;
	protected int  m_sourceHighPort;
	protected int  m_destinationLowPort;
	protected int  m_destinationHighPort;
	protected long m_packetCount;
	protected long m_byteCount;
	protected short m_TOSand;
	protected short m_TOSxor;
  	protected boolean m_affectedToService;
  	protected boolean m_isApplied;

	protected boolean mb_antiSpoofingRule;
	protected boolean mb_logDeniedPacketsRule;
/**
 * InputRule constructor.
 */
public Rule()
{
	super();
	m_sourceLowPort = 0;
	m_sourceHighPort = 65535;
	m_destinationLowPort = 0;
	m_destinationHighPort = 65535;
}
public Rule(String raw)
{
	super();

	StringTokenizer st = new StringTokenizer(raw, " \t\r\n/");

	m_ruleType = st.nextToken();
	m_sourceAddress = new IPAddress(Long.parseLong(st.nextToken(), 16));
	String sourceMaskArrowDestAddress = st.nextToken();
	StringTokenizer sad = new StringTokenizer(sourceMaskArrowDestAddress, "->");
	m_sourceMask = new IPMask(Long.parseLong(sad.nextToken(), 16));
	m_destinationAddress = new IPAddress(Long.parseLong(sad.nextToken(), 16));
	m_destinationMask = new IPMask(Long.parseLong(st.nextToken(), 16));
	m_interfaceName = st.nextToken();
	if (m_interfaceName.equals("-")) m_interfaceName = "all";

	m_flags = Integer.parseInt(st.nextToken(), 16);
	m_flags2 = Integer.parseInt(st.nextToken(), 16);

	m_protocol = Integer.parseInt(st.nextToken());
	for (int i=0; i < 4; i++) st.nextToken();

	String sourcePortsRange = st.nextToken();
	StringTokenizer sts = new StringTokenizer(sourcePortsRange, "-");
	m_sourceLowPort = Integer.parseInt(sts.nextToken());
	m_sourceHighPort = Integer.parseInt(sts.nextToken());
	
	String destinationPortsRange = st.nextToken();
	StringTokenizer std = new StringTokenizer(destinationPortsRange, "-");
	m_destinationLowPort = Integer.parseInt(std.nextToken());
	m_destinationHighPort = Integer.parseInt(std.nextToken());

	for (int i=0; i < 5; i++) st.nextToken();
	m_policy = st.nextToken();
}
public boolean destinationNetworkMatch(IPInterface ipi)
{
  if ((m_destinationAddress.getLong() & m_destinationMask.getLong()) == (ipi.getAddress().getLong() & ipi.getMask().getLong()))
	return true;

  return false;
}
public String getDestinationPortsString()
{
  if ((m_destinationLowPort == 0) && (m_destinationHighPort == 0)) m_destinationHighPort = 65535;

  if (m_destinationLowPort != m_destinationHighPort)
	return "" + m_destinationLowPort + ":" + m_destinationHighPort;

  return Linux22.getServiceName(m_destinationLowPort, getProtocolString());
}
public String getDestinationPortsStringForDisplay()
{
  if ((m_destinationLowPort == 0) && (m_destinationHighPort == 0)) return "all";
  if ((m_destinationLowPort == 0) && (m_destinationHighPort == 65535)) return "all";

  if (m_destinationLowPort != m_destinationHighPort)
	return "" + m_destinationLowPort + ":" + m_destinationHighPort;

  String res = Linux22.getServiceName(m_destinationLowPort, getProtocolString());
  try
  {
	int test = Integer.parseInt(res);
  }
  catch(NumberFormatException nfe)
  {
	res += " (" + m_destinationLowPort + ")";
  }

  return res;
}
public String getProtocolString()
{
  String res="";
	switch (m_protocol)
	{
	case Linux22.IPPROTO_TCP:
		res += ("tcp");
		break;
	case Linux22.IPPROTO_UDP:
		res += ("udp");
		break;
	case Linux22.IPPROTO_ICMP:
		res += ("icmp");
		break;
	case Linux22.IPPROTO_IP:
		res += ("all");
	}
  return res;
}
public String getRuleType()
{
	return m_ruleType;
}
public String getSourcePortsString()
{
  if ((m_sourceLowPort == 0) && (m_sourceHighPort == 0)) m_sourceHighPort = 65535;

  if (m_sourceLowPort != m_sourceHighPort)
	return "" + m_sourceLowPort + ":" + m_sourceHighPort;

  return Linux22.getServiceName(m_sourceLowPort, getProtocolString());
}
public String getSourcePortsStringForDisplay()
{
  if ((m_sourceLowPort == 0) && (m_sourceHighPort == 0)) return "all";
  if ((m_sourceLowPort == 0) && (m_sourceHighPort == 65535)) return "all";

  if (m_sourceLowPort != m_sourceHighPort)
	return "" + m_sourceLowPort + ":" + m_sourceHighPort;

  String res = Linux22.getServiceName(m_sourceLowPort, getProtocolString());
  try
  {
	int test = Integer.parseInt(res);
  }
  catch(NumberFormatException nfe)
  {
	res += " (" + m_sourceLowPort + ")";
  }

  return res;
}
public boolean isDestinationPort(int dport)
{
	if ((dport >= m_destinationLowPort) && (dport <= m_destinationHighPort)) return true;
	return false;
}
public boolean isForwardRule()
{
	return m_ruleType.equals("forward");
}
public boolean isInputRule()
{
	return m_ruleType.equals("input");
}
public boolean isOutputRule()
{
	return m_ruleType.equals("output");
}
public boolean isSourcePort(int sport)
{
	if ((sport >= m_sourceLowPort) && (sport <= m_sourceHighPort)) return true;
	return false;
}
public boolean portsMatch(Rule r)
{
	if (r.m_destinationLowPort == m_sourceLowPort)
	if (r.m_destinationHighPort == m_sourceHighPort)
	if (r.m_sourceLowPort == m_destinationLowPort)
	if (r.m_sourceHighPort == m_destinationHighPort)
	return true;

	return false;
}
public boolean sourceNetworkMatch(IPInterface ipi)
{
  if ((m_sourceAddress.getLong() & m_sourceMask.getLong()) == (ipi.getAddress().getLong() & ipi.getMask().getLong()))
	return true;

  return false;
}
public String toString()
{
	String res = m_ruleType;
	res += " " + m_policy;
	res += " SRC[" + m_sourceAddress.toString() + "/" + m_sourceMask.toString() + "]";
	res += " DST[" + m_destinationAddress.toString() + "/" + m_destinationMask.toString() + "]";
	res += " IF[" + m_interfaceName + "]";
	res += " PROT[" + getProtocolString() + "]";
	res += " SPORT[";
	if (m_sourceLowPort != m_sourceHighPort)
		res += m_sourceLowPort + ":" + m_sourceHighPort;
	else
		res += getSourcePortsStringForDisplay();
	res += "]";
	res += " DPORT[";
	if (m_destinationLowPort != m_destinationHighPort)
		res += m_destinationLowPort + ":" + m_destinationHighPort;
	else
		res += getDestinationPortsStringForDisplay();
	res += "]";
	return res;
}
}
