package fwadmin;

/*

  JFwadmin v 0.82

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.util.Enumeration;
import java.util.TreeSet;
import java.util.Iterator;

public class Service
{
  Rule m_ir;
  Rule m_or;
  private String m_serviceComment;

  public Service(Rule ir, Rule or)
  {
	m_ir = ir;
	m_or = or;
  }    
  public Service(Rule ir, Rule or, String comment)
  {
	m_ir = ir;
	m_or = or;
	m_serviceComment = comment;
  }    
  public void addComment(String comment)
  {
	if (comment.equals("")) return;
	if (m_serviceComment == null)
	{
	  m_serviceComment = comment;
	  return;
	}
	m_serviceComment += "; " + comment;
  }    
public static void addInputService(Rule ir, Rule or)
{
	addInputService(ir, or, "");
}                                      
  public static void addInputService(Rule ir, Rule or, String comment)
  {
	InputService is = new InputService(ir, or);
	is.addComment(comment);
	ir.m_affectedToService = true;
	or.m_affectedToService = true;
	if (!ir.m_isApplied) is.addComment("not applied");
	InputService.inputServicesVector.addElement(is);

	// Try to set a good comment and ordering type on this service
	for (Enumeration e = IPInterface.interfacesHashtable.elements(); e.hasMoreElements(); )
	{
	  IPInterface ipi = (IPInterface) e.nextElement();
	  if (ir.m_policy.equals("DENY"))
	  if (! ipi.getName().equals(ir.m_interfaceName))
	  //if (! ipi.getKind().equals(IPFWConstants.networkKindInternet))
	  if (ir.sourceNetworkMatch(ipi))
	  {
		is.addComment("anti spooffing");
		ir.mb_antiSpoofingRule = true;
		or.mb_antiSpoofingRule = true;
	  }
	}
	if (((ir.m_flags & Linux22.FLAG_LOGGING) > 0) && (! ir.mb_antiSpoofingRule))
	{
	 	if (ir.m_sourceAddress.getLong() == 0L)
	 	if (ir.m_sourceMask.getLong() == 0L)
	 	if (ir.m_destinationAddress.getLong() == 0L)
	 	if (ir.m_destinationMask.getLong() == 0L)
	 	if (ir.m_sourceLowPort == 0)
	 	if (ir.m_sourceHighPort == 65535)
	 	if (ir.m_destinationLowPort == 0)
	 	if (ir.m_destinationHighPort == 65535)
	 	{
	    	is.addComment("log denied packets");
			ir.mb_logDeniedPacketsRule = true;
			or.mb_logDeniedPacketsRule = true;
	 	}
	 	if (! ir.mb_logDeniedPacketsRule)
	 	{
	    	is.addComment("logged");
	 	}
	}
	if ((ir.m_protocol == Linux22.IPPROTO_ICMP) ||
		((ir.m_destinationLowPort == 0) && (ir.m_destinationHighPort == 65535) && (ir.m_sourceLowPort == 0) && (ir.m_sourceHighPort == 65535) && ir.portsMatch(or)))
	{
	  is.addComment("Is also output rule");
	}
  }                                                            
  public static void addOutputService(Rule ir, Rule or)
  {
	  addOutputService(ir, or, "");
  }                
  public static void addOutputService(Rule ir, Rule or, String comment)
  {
	OutputService os = new OutputService(ir, or);
	os.addComment(comment);
	ir.m_affectedToService = true;
	or.m_affectedToService = true;
	if (!or.m_isApplied) os.addComment("not applied");
	OutputService.outputServicesVector.addElement(os);

	if ((or.m_flags & Linux22.FLAG_LOGGING) > 0)
	{
	  os.addComment("logged");
	}
	if (((ir.m_flags & Linux22.FLAG_TCP_SYN) > 0) && ((ir.m_flags2 & Linux22.FLAG2_NO_TCP_SYN) > 0))
	{
	  os.addComment("drop incoming tcp syn");
	}
  }                      
  public String getComment()
  {
	if (m_serviceComment == null) return "";
	return m_serviceComment;
  }  
public static void initServices()
{
	int pass = 0;
	do
	{
		OutputService.outputServicesVector.removeAllElements();
		InputService.inputServicesVector.removeAllElements();
		Iterator it1 = Rules.rulesTreeSet.iterator();
		while (it1.hasNext())
		{
			Rule ir = (Rule) it1.next();
			if (!ir.isInputRule())
				continue;
				
			Iterator it2 = Rules.rulesTreeSet.iterator();
			while (it2.hasNext())
			{
				Rule or = (Rule) it2.next();
				if (!or.isOutputRule())
					continue;
				if (ir.m_interfaceName.equals(or.m_interfaceName))
				if (ir.m_protocol == or.m_protocol)
				if (ir.portsMatch(or))
				if (ir.m_policy.equals(or.m_policy))
				if (ir.m_sourceAddress.equals(or.m_destinationAddress))
				if (ir.m_destinationAddress.equals(or.m_sourceAddress))
				if (ir.m_sourceMask.equals(or.m_destinationMask))
				if (ir.m_destinationMask.equals(or.m_sourceMask))
				{
					if (or.m_affectedToService)
					{
						//System.out.println("Rule matches taken rule : ");
						//System.out.println(ir.toString());
						//System.out.println(or.toString());
					}
					else
					// Rules match
					if ((ir.m_sourceHighPort - ir.m_sourceLowPort) >= (ir.m_destinationHighPort - ir.m_destinationLowPort))
					{
						// Hope this is an input service
						addInputService(ir, or);
					}
					else
					{
						// Hope this is an output service
						addOutputService(ir, or);
					}
				}
			}
		}
		
		pass++;
		if (pass == 1)
		{
			Rules.rulesTreeSet = Linux22.sortRules(Rules.rulesTreeSet);
			Iterator it2 = Rules.rulesTreeSet.iterator();
			while (it2.hasNext())
			{
				Rule ir = (Rule) it2.next();
				ir.m_affectedToService = false;
			}
		}
	}
	while (pass < 2);
}
}
